%%%%%%%%%%%%%%%%%%%%%%%% A Model with non-tradable goods and mining sector
%%%%%%%%%%%%%this program is Bayesian estimation

% Author: Gunbileg Ganbayar

close all;
warning off
%%
%%%%%%%%%%%%%%%%%%%%%%%Endogenous Variables %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
var
c               % consumption
rk              % rental rate of capital
w               % wage
h               % total hours
hT              % T hours
hN              % N hours
yT              % T output
yN              % N output
yO				% Mining output 
k               % total capital
kT              % T capital
kN              % N capital
q               % Tobin Q
i               % investement
lambda          % marginal utility of consumption
d               % debt
p               % CPI (or real exchange rate)
pN              % N price
pO				% Mining price 
tb              % trade balance
gdp             % gross domestic product
g               % government spending     
aT              % T TFP
aN              % N TFP
rz              % foreign rate
ac 				% preference shock
Z				% auxiliary variable for checking market clearing

// observed variables
NX_GDP GDP_Growth G_Growth M_GDP_Growth M_Price_Growth Cons_Growth Foreign_Rate

% log variables to have IRFs in percentage deviations from the ss
clog
wlog 
hlog
hTlog
hNlog
yNlog
yTlog
gdplog
klog
kNlog
kTlog
ilog
pNlog
plog
dy    
dlog
;
%%
%%%%%%%%%%%%%%%%%%%%%%%Exogenous Variables%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varexo 
vaT  % tradable productivity shock
vaN  % non-tradable productivity shock
vg   % public spending shock
vp   % foreign monetary policy shock

vpO  % commodity price shock
vyO	 % mining production shock
vc 	 % preference shock
;  
    
%%
%%%%%%%%%%%%%%%%%%%%%%%Parameters%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parameters

beta 
alphaT
alphaN
delta
sigma
phi 
kappaL
eta
gamma
kNss
kTss
hNss
aTss
rzss
dss
gss
pNss
kappaD
kappaI
rhoaT
rhoaN
rhog
rhop 
yOss 
rhoyO 
rhopO 
rhoc
O 
tO
;


load par;  % load mat file created in console
for jj=1:length(M_.param_names)
set_param_value(M_.param_names{jj},eval(M_.param_names{jj})); 
end; 

%%
%%%%%%%%%%%%%%%%%%%%%%%Non-Linear Model%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model;
% Households
lambda=ac*(c-kappaL/(1+phi)*h^(1+phi))^-sigma;  
1=beta*lambda(1)/lambda*rz*p/p(1)+kappaD*(d-dss);
1=beta*lambda(1)/lambda*(rk(1)+(1-delta)*q(1))/q;
kappaL*h^(phi)=w;        
k=(1-delta)*k(-1)+(1-kappaI/2*(i/i(-1)-1)^2)*i;
1=q*(1-kappaI/2*(i/i(-1)-1)^2-kappaI*(i/i(-1)-1)*i/i(-1))+kappaI*beta*lambda(1)/lambda*q(1)*(i(1)/i-1)*(i(1)/i)^2;    
% Firms
yT=aT*kT^alphaT*hT^(1-alphaT);
yN=aN*kN^alphaN*hN^(1-alphaN);
(1-alphaT)*yT=w*hT*p;
(1-alphaN)*yN=w*hN*p/pN;
alphaT*yT=rk*kT*p;
alphaN*yN=rk*kN*p/pN;
% Market clearing
yN=(1-gamma)*(pN/p)^(-eta)*(c+i+g-yO*tO);
k(-1)=kT+kN;
h=hT+hN;
gdp=c+i+g-yO*tO+tb
;
//Mining output and price
yO=yOss*(yO(-1)/yOss)^rhoyO*exp(vyO);
pO=pO(-1)^rhopO*exp(vpO);
//
% Prices
p^(1-eta)=(1-gamma)*pN^(1-eta)+gamma;
% Shocks
log(aT)=(1-rhoaT)*log(aTss)+rhoaT*log(aT(-1))+vaT;
log(aN)=rhoaN*log(aN(-1))+vaN;  
log(g-yO*tO)=(1-rhog)*log(gss-yOss*tO)+rhog*log(g(-1)-yO(-1)*tO)+vg;  
rz=(1-rhop)*1/beta+rhop*rz(-1)+vp;
log(ac)=rhoc*log(ac(-1))+vc;
% Auxiliary variables
gdp=(pN*yN+yT+pO*yO)/p;
tb=-d/p+rz(-1)*d(-1)/p+kappaD/(2*p)*(d-dss)^2;
// check
Z=yT-(gamma*(p)^(eta)*(c+i+g-yO*tO)+p*tb); 
dy=d/(p*gdp);
% Log variables
clog=log(c); 
wlog=log(w); 
hlog=log(h);  
hNlog=log(hN); 
hTlog=log(hT);
klog=log(k); 
kNlog=log(kN); 
kTlog=log(kT);
ilog=log(i); 
yTlog=log(yT);
yNlog=log(yN); 
gdplog=log(gdp);
plog=log(p); 
pNlog=log(pN);
dlog=log(d);
//// observation equation
NX_GDP=tb/gdp;
GDP_Growth=gdp/gdp(-1)-1;
G_Growth=g/g(-1)-1;
M_GDP_Growth=yO/yO(-1)-1;
M_Price_Growth=pO/pO(-1)-1;
Cons_Growth=c/c(-1)-1;
Foreign_Rate=rz-1;

end;

%% Steady State values are estimated in seperate file, console_mining.mat

initval;
aT=aTss;
kT=kTss;
kN=kNss;
pN=pNss;
hN=hNss;
// mining steady state
pO=1;
yO=yOss;
// steady state preference shock
ac=1;
// 
h=1/3;
rz=1/beta;
q=1;
rk=1/beta-(1-delta); 
p=pss;

k=kTss+kNss;
i=delta*k;
hT=hTss;
yT=aT*kT^alphaT*hT^(1-alphaT);
yN=kN^alphaN*hN^(1-alphaN);
gdp=(pN*yN+yT)/(p-O);
d=dss;
tb=d/p*(1/beta-1);
g=gss;
c=gdp-i-g+yO*tO-tb;
w=(1-alphaT)*yT/(hT*p);
lambda=(c-kappaL/(1+phi)*h^(1+phi))^-sigma; 
aN=1;
clog=log(c);  
wlog=log(w); 
hlog=log(h);  
hNlog=log(hN); 
hTlog=log(hT);
klog=log(k); 
kNlog=log(kN); 
kTlog=log(kT);
ilog=log(i); 
yTlog=log(yT);
yNlog=log(yN); 
gdplog=log(gdp);
plog=log(p); 
pNlog=log(pN);
dy=d/(p*gdp);
dlog=log(d);
Z=0;
end;

steady;
check;

%% Shocks scaled unit standard deviation
shocks;
var vaT=1;     
var vaN=1;     
var vg=1;    
var vp=1;
var vyO=1;
var vpO=1;
var vc=1;    
end; 

// Observed variables
varobs NX_GDP GDP_Growth G_Growth M_GDP_Growth M_Price_Growth Foreign_Rate
;

estimated_params;
rhoaN, beta_pdf, 0.6, 0.1;
rhoaT, beta_pdf, 0.6, 0.1;
rhoc, beta_pdf, 0.6, 0.1;
rhog, beta_pdf, 0.6, 0.1;
rhop, beta_pdf, 0.6, 0.1;
rhoyO, beta_pdf, 0.6, 0.1;
rhopO, beta_pdf, 0.6, 0.1;
stderr vaT, inv_gamma_pdf, 0.01, inf;
stderr vaN, inv_gamma_pdf, 0.01, inf;
stderr vg, inv_gamma_pdf, 0.01, inf;
stderr vp, inv_gamma_pdf, 0.01, inf;
stderr vyO, inv_gamma_pdf, 0.01, inf;
stderr vpO, inv_gamma_pdf, 0.01, inf;
stderr vc,  inv_gamma_pdf, 0.01, inf;
end;


%% estimation and bayesian IRF
estimation(datafile=data,mh_replic=10000,mh_nblocks=2,
mh_drop=0.1,mh_jscale=2.5,conf_sig=.99,bayesian_irf,irf=100) 
clog wlog hlog hNlog hTlog klog kNlog kTlog ilog yTlog yNlog gdplog plog pNlog dlog dy;

%% stoch_simul;